#ifndef _IPCONTACT_APP_H
#define _IPCONTACT_APP_H

#include <asterisk.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/version.h>

#include <mysql/mysql.h>

#define IPCONTACT_ADDONS_VERSION "1.4.12"
#define IPCONTACT_CONFDIR "/etc/asterisk"

#define	IPC_EXTEN_NUMBER "number"
#define	IPC_EXTEN_INFO "info"
#define	IPC_EXTEN_PROTO "protocol"
#define IPC_EXTEN_CHANNEL "channel"
#define IPC_EXTEN_VM "voicemail"
#define	IPC_EXTEN_VMFLAG "vm_flag"
#define	IPC_EXTEN_MONITOR "monitor"

#define IPC_PROFILE_LOCAL "local"
#define IPC_PROFILE_CELLULAR "cellular"
#define IPC_PROFILE_LONGDIST "longdist"
#define IPC_PROFILE_INTER "inter"

#define JASTSERV_PORT 6038
#define JASTSERV_SSL_PORT 6037
#define JASTSERV_RECONNECT_TIMEOUT 5
#define JASTSERV_SEND_PREFIX "+ TX"
#define JASTSERV_RECV_PREFIX "+ RX"

/* Maximum variable lengths */
// AST_MAX_APP 32 - defined in pbx.h
// AST_MAX_EXTENSION 80 - defined in channel.h
// AST_MAX_CONTEXT 80 - defined in channel.h
#define IPC_MAX_LEN 128
#define IPC_QUERY_LEN 256
#define IPC_QUERYLONG_LEN 1024
#define IPC_MYSQL_PING_TIME 10

/* Macros */
#define CHAN_REAL_EXTEN(ch)  (ast_strlen_zero(ch->macroexten)?ch->exten:ch->macroexten)
#define IS_INCOMING_CALL(ch) ast_true(pbx_builtin_getvar_helper(chan, "INCOMING_CALL"))
#define IS_OUTGOING_CALL(ch) ast_true(pbx_builtin_getvar_helper(chan, "OUTGOING_CALL"))
#define IS_INCOMINGCELL_CALL(ch) ast_true(pbx_builtin_getvar_helper(ch, "INCOMING_CELL"))

/* Version conditionals */
#ifdef ASTERISK_VERSION_NUM

#if (ASTERISK_VERSION_NUM >= 10800)
#define HAS_AST18
#define HAS_XMLDOC
#warning "Using Asterisk 1.8 API"
#else 
#if (ASTERISK_VERSION_NUM >= 10600)
#define HAS_AST16
#if (ASTERISK_VERSION_NUM >= 10602)
// XML doc introduced on 1.6.2
#define HAS_XMLDOC
#endif
#warning "Using Asterisk 1.6 API"
#else 
#if (ASTERISK_VERSION_NUM >= 10400)
#warning "Using Asterisk 1.4 API"
#else
#error "Asterisk 1.2 is not supported!"
#endif
#endif
#endif

#else
#error "ASTERISK_VERSION_NUM is not defined. Please install asterisk-devel package"
#endif

/* Asterisk argument delimiter, will be defined someday (see https://reviewboard.asterisk.org/r/79/) */
#ifndef AST_STANDARD_APP_DELIM
#if defined(HAS_AST16) || defined(HAS_AST18)
#define AST_STANDARD_APP_DELIM ","
#else
#define AST_STANDARD_APP_DELIM "|"
#endif
#endif

#ifdef HAS_AST18
#define CALLERID_NUM(ch) ch->caller.id.number.str
#define CALLERID_NAME(ch) ch->caller.id.name.str
#define CALLERID_DNID(ch) ch->dialed.number.str
#else
#define CALLERID_NUM(ch) ch->cid.cid_num
#define CALLERID_NAME(ch) ch->cid.cid_name
#define CALLERID_DNID(ch) ch->cid.cid_dnid
#endif

const char *IPC_EXTEN_TREE = "ipcontact/extension";
const char *IPC_HUNT_TREE = "ipcontact/huntgroup";

/* res_ipcontact configuration */
struct ast_ipcontact_conf {
	char dbhost[AST_MAX_APP];
	char dbname[AST_MAX_APP];
	char dbuser[AST_MAX_APP];
	char dbpass[AST_MAX_APP];
	unsigned int dbport;
	char dbsock[AST_MAX_APP];
	char tables[IPC_MAX_LEN];
	char jhost[AST_MAX_APP];
	char juser[AST_MAX_APP];
	char jpass[AST_MAX_APP];
	unsigned int jldap;
	unsigned int jdebug;
};

/* "Custom table" list */
struct ast_ipcontact_table
{
	char table[AST_MAX_APP];
	AST_LIST_ENTRY(ast_ipcontact_table) list; /*   Next table in list */
};

/* VPBX list */
struct ast_ipcontact_vpbx {
	int number;
	int family;
	char phone[AST_MAX_EXTENSION];
	char name[AST_MAX_EXTENSION];
	char mainmenu[AST_MAX_CONTEXT];
	char operator[AST_MAX_CONTEXT];
	int local_timeout;
	int ext_timeout;
	int monitor;
	/*   Next virtual pbx in list */
	AST_LIST_ENTRY(ast_ipcontact_vpbx) list;
};

/* Extension list */
struct ast_ipcontact_extension {
	char number[AST_MAX_EXTENSION];
	/*   Next extension in list */
	AST_LIST_ENTRY(ast_ipcontact_extension) list;
};

/* Agents list */
struct ast_ipcontact_agent {
	char username[AST_MAX_EXTENSION];
	/*   Next extension in list */
	AST_LIST_ENTRY(ast_ipcontact_agent) list;
};

/**** MySQL functions ****/
MYSQL *ipc_sql_reconnect(void);
void ipc_sql_disconnect(void);

/* ipc_sql_exec_query(char* query):
   Recibe un string con la consulta a realizar en la base de datos
   y devuelve un puntero a MYSQL_RES con el resultado o NULL si se
   produjo un error.
*/
MYSQL_RES *ipc_sql_exec_query(char *query);

int ipc_mutex_lock(void);
int ipc_mutex_unlock(void);

struct ast_ipcontact_vpbx *get_virtual_pbx(int vpbx);
int get_exten_profile(const char *exten, const char *profile);

int is_number(const char *num);

const char *ast_channel_type(struct ast_channel *chan);

#endif
